@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2006-2020 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -24,20 +24,22 @@ from mailman.database.model import Model
 from mailman.database.transaction import dbconnection
 from mailman.database.types import Enum, SAUnicode, SAUnicodeLarge
 from mailman.interfaces.action import Action, FilterAction
-from mailman.interfaces.address import IAddress
+from mailman.interfaces.address import IAddress, InvalidEmailAddressError
 from mailman.interfaces.archiver import ArchivePolicy
 from mailman.interfaces.autorespond import ResponseAction
+from mailman.interfaces.bans import IBanManager
 from mailman.interfaces.bounce import UnrecognizedBounceDisposition
 from mailman.interfaces.digests import DigestFrequency
 from mailman.interfaces.domain import IDomainManager
 from mailman.interfaces.languages import ILanguageManager
 from mailman.interfaces.mailinglist import (
-    DMARCMitigateAction, IAcceptableAlias, IAcceptableAliasSet,
-    IHeaderMatch, IHeaderMatchList, IListArchiver, IListArchiverSet,
-    IMailingList, Personalization, ReplyToMunging, SubscriptionPolicy)
+    ArchiveRenderingMode, DMARCMitigateAction, IAcceptableAlias,
+    IAcceptableAliasSet, IHeaderMatch, IHeaderMatchList, IListArchiver,
+    IListArchiverSet, IMailingList, Personalization, ReplyToMunging,
+    SubscriptionPolicy)
 from mailman.interfaces.member import (
-    AlreadySubscribedError, MemberRole, MissingPreferredAddressError,
-    SubscriptionEvent)
+    AlreadySubscribedError, MemberRole, MembershipIsBannedError,
+    MissingPreferredAddressError, SubscriptionEvent)
 from mailman.interfaces.mime import FilterType
 from mailman.interfaces.nntp import NewsgroupModeration
 from mailman.interfaces.user import IUser
@@ -54,6 +56,7 @@ from sqlalchemy import (
     LargeBinary, PickleType)
 from sqlalchemy.event import listen
 from sqlalchemy.ext.hybrid import hybrid_property
+from sqlalchemy.ext.mutable import MutableList
 from sqlalchemy.orm import relationship
 from sqlalchemy.orm.exc import NoResultFound
 from zope.component import getUtility
@@ -98,7 +101,7 @@ class MailingList(Model):
     # Attributes which are directly modifiable via the web u/i.  The more
     # complicated attributes are currently stored as pickles, though that
     # will change as the schema and implementation is developed.
-    accept_these_nonmembers = Column(PickleType)       # XXX
+    accept_these_nonmembers = Column(MutableList.as_mutable(PickleType))  # XXX
     admin_immed_notify = Column(Boolean)
     admin_notify_mchanges = Column(Boolean)
     administrivia = Column(Boolean)
@@ -117,13 +120,13 @@ class MailingList(Model):
     collapse_alternatives = Column(Boolean)
     convert_html_to_plaintext = Column(Boolean)
     # Bounces.
-    bounce_info_stale_after = Column(Interval)                   # XXX
+    bounce_info_stale_after = Column(Interval)
     bounce_matching_headers = Column(SAUnicode)                  # XXX
-    bounce_notify_owner_on_disable = Column(Boolean)             # XXX
-    bounce_notify_owner_on_removal = Column(Boolean)             # XXX
-    bounce_score_threshold = Column(Integer)                     # XXX
-    bounce_you_are_disabled_warnings = Column(Integer)           # XXX
-    bounce_you_are_disabled_warnings_interval = Column(Interval)  # XXX
+    bounce_notify_owner_on_disable = Column(Boolean)
+    bounce_notify_owner_on_removal = Column(Boolean)
+    bounce_score_threshold = Column(Integer)
+    bounce_you_are_disabled_warnings = Column(Integer)
+    bounce_you_are_disabled_warnings_interval = Column(Interval)
     forward_unrecognized_bounces_to = Column(
         Enum(UnrecognizedBounceDisposition))
     process_bounces = Column(Boolean)
@@ -141,14 +144,14 @@ class MailingList(Model):
     digest_send_periodic = Column(Boolean)
     digest_size_threshold = Column(Float)
     digest_volume_frequency = Column(Enum(DigestFrequency))
-    discard_these_nonmembers = Column(PickleType)
+    discard_these_nonmembers = Column(MutableList.as_mutable(PickleType))
     emergency = Column(Boolean)
     encode_ascii_prefixes = Column(Boolean)
     first_strip_reply_to = Column(Boolean)
     forward_auto_discards = Column(Boolean)
     gateway_to_mail = Column(Boolean)
     gateway_to_news = Column(Boolean)
-    hold_these_nonmembers = Column(PickleType)
+    hold_these_nonmembers = Column(MutableList.as_mutable(PickleType))
     info = Column(SAUnicode)
     linked_newsgroup = Column(SAUnicode)
     max_days_to_hold = Column(Integer)
@@ -169,7 +172,7 @@ class MailingList(Model):
     posting_pipeline = Column(SAUnicode)
     _preferred_language = Column('preferred_language', SAUnicode)
     display_name = Column(SAUnicode)
-    reject_these_nonmembers = Column(PickleType)
+    reject_these_nonmembers = Column(MutableList.as_mutable(PickleType))
     reply_goes_to_list = Column(Enum(ReplyToMunging))
     reply_to_address = Column(SAUnicode)
     require_explicit_destination = Column(Boolean)
@@ -184,6 +187,8 @@ class MailingList(Model):
     topics_bodylines_limit = Column(Integer)
     topics_enabled = Column(Boolean)
     unsubscription_policy = Column(Enum(SubscriptionPolicy))
+    usenet_watermark = Column(Integer)
+    archive_rendering_mode = Column(Enum(ArchiveRenderingMode))
     # ORM relationships.
     header_matches = relationship(
         'HeaderMatch', backref='mailing_list',
@@ -481,17 +486,26 @@ class MailingList(Model):
         return member is not None
 
     @dbconnection
-    def subscribe(self, store, subscriber, role=MemberRole.member):
+    def subscribe(self, store, subscriber, role=MemberRole.member,
+                  send_welcome_message=None):
         """See `IMailingList`."""
         member, email = self._get_subscriber(store, subscriber, role)
+        test_email = email or subscriber.lower()
+        # Allow list posting address only for nonmember role.
+        if (test_email == self.posting_address and
+                role != MemberRole.nonmember):
+            raise InvalidEmailAddressError('List posting address not allowed')
         if member is not None:
             raise AlreadySubscribedError(self.fqdn_listname, email, role)
+        if IBanManager(self).is_banned(test_email):
+            raise MembershipIsBannedError(self, test_email)
         member = Member(role=role,
                         list_id=self._list_id,
                         subscriber=subscriber)
         member.preferences = Preferences()
         store.add(member)
-        notify(SubscriptionEvent(self, member))
+        notify(SubscriptionEvent(
+            self, member, send_welcome_message=send_welcome_message))
         return member
 
 
@@ -641,6 +655,7 @@ class HeaderMatch(Model):
     header = Column(SAUnicode)
     pattern = Column(SAUnicode)
     chain = Column(SAUnicode, nullable=True)
+    tag = Column(SAUnicode, nullable=True)
 
     def __init__(self, **kw):
         position = kw.pop('position', None)
@@ -709,7 +724,7 @@ class HeaderMatchList:
         del self._mailing_list.header_matches[:]
 
     @dbconnection
-    def append(self, store, header, pattern, chain=None):
+    def append(self, store, header, pattern, chain=None, tag=None):
         header = header.lower()
         existing = store.query(HeaderMatch).filter(
             HeaderMatch.mailing_list == self._mailing_list,
@@ -725,19 +740,20 @@ class HeaderMatchList:
         header_match = HeaderMatch(
             mailing_list=self._mailing_list,
             header=header, pattern=pattern, chain=chain,
-            position=last_position + 1)
+            position=last_position + 1, tag=tag)
         store.add(header_match)
         store.expire(self._mailing_list, ['header_matches'])
 
     @dbconnection
-    def insert(self, store, index, header, pattern, chain=None):
-        self.append(header, pattern, chain)
+    def insert(self, store, index, header, pattern, chain=None, tag=None):
+        self.append(header, pattern, chain, tag)
         # Get the header match that was just added.
         header_match = store.query(HeaderMatch).filter(
             HeaderMatch.mailing_list == self._mailing_list,
             HeaderMatch.header == header.lower(),
             HeaderMatch.pattern == pattern,
-            HeaderMatch.chain == chain).one()
+            HeaderMatch.chain == chain,
+            HeaderMatch.tag == tag).one()
         header_match.position = index
         store.expire(self._mailing_list, ['header_matches'])
 
@@ -806,3 +822,24 @@ class HeaderMatchList:
                 ).order_by(HeaderMatch.position)):
             match._position = position
         store.expire(self._mailing_list, ['header_matches'])
+
+    @dbconnection
+    def get_by_tag(self, store, tag):
+        # Get all the header matches that correspond to a single tag.
+        entries = store.query(HeaderMatch).filter(
+            HeaderMatch.mailing_list == self._mailing_list,
+            HeaderMatch.tag == tag
+            ).order_by(HeaderMatch.position)
+        yield from entries
+
+    @dbconnection
+    def filter(self, store, header=None, chain=None, tag=None):
+        entries = store.query(HeaderMatch).filter(
+            HeaderMatch.mailing_list == self._mailing_list)
+        if tag:
+            entries = entries.filter(HeaderMatch.tag == tag)
+        if header:
+            entries = entries.filter(HeaderMatch.header == header)
+        if chain:
+            entries = entries.filter(HeaderMatch.chain == chain)
+        yield from entries.order_by(HeaderMatch.position)
